from django.shortcuts import render, render_to_response
from django import forms
from meetup import api as meetupapi
import math
import pprint
from datetime import datetime
from django.conf import settings

# Store the apikey in a conf file to avoid exposing it in git repo
# you can use just "apikey=SECRETKEY" instead of this malarkey
apikey=open("/home/sumner/.dojo/meetupapikey.txt", "r").readlines()[0].strip()

# Estimated number of pizzas per person who actually shows up
def pizzas(e):
    slice_total = e*.6*2.5
    pizza_total = math.ceil(slice_total/8)
    return pizza_total

# Figure out how many people are likely to attend, based
# on past actual attendance vs. rsvp numbers for the group
# There's a floor of 30% to avoid really bad data issues
def find_attendance_factor(events):
    total = 0
    rsvp = 0
    for res in events.results:
        if res['headcount']:
            total += res['headcount']
            rsvp += res['yes_rsvp_count']
    estimate = total*1.0/rsvp
    if estimate<.3: return .3
    else: return estimate

# Fetch the Meetup data from the API, including historical data
# Get all the data massaged in a format for bokeh graphing to use
def graph_pizza_by_group(groupurl):
    client = meetupapi.Client(apikey)
    group_info = client.GetGroup({'urlname': groupurl})
    events = client.GetEvents({'group_id': group_info.id, 'status':'past,upcoming'})
    # Meetup doesn't use Unix timestamps, it uses milliseconds since the epoch
    # So we have to divide by 1000 before parsing them.
    for e in events.results:
        e["date"] = datetime.fromtimestamp(e["time"]/1000)

    est_factor = find_attendance_factor(events)

    # Build a set of lists for graphing purposes....
    # Dates
    datelist = [ res["date"] for res in events.results]
    # labels in the form of "Meeting Name (12 15 2016)"
    labellist = [ "%s (%s)"%(res['name'], res['date'].strftime("%b %d %Y")) for res in events.results ]
    # Number of RSVPs for the meeting
    rsvplist = [ res['yes_rsvp_count'] for res in events.results ]
    # Actual attendees for the meeting
    attendlist = [ res['headcount'] for res in events.results ]
    # Estimated # of attendees based on the RSVPs and find_attendance_factor
    estimatelist = [est_factor*x for x in rsvplist]
    # Pizza count bast on estimated attendees and pizzas per person
    pizzacount = [pizzas(x) for x in estimatelist]
    return (datelist, labellist, rsvplist, attendlist, estimatelist, pizzacount)


# Minimal form for the search page https://sumnerhayes.com/dojo/meetupsearch
class NameForm(forms.Form):
    group_name = forms.CharField(label='Meetup Group:', max_length=100)

# Main Django entry point for the search results (graph) page 
# https://sumnerhayes.com/dojo/meetup/?group_name=Data
# This would go in your urls.py in Django setup
def meetup(request):
    stuff = graph_pizza_by_group(request.GET['group_name'])
    group = 'Data-Visualization-DC'
    datelist, labellist, rsvplist, attendlist, estimatelist, pizzacount = graph_pizza_by_group(group)
    return pizza_tracker(request, datelist, rsvplist, attendlist, estimatelist, pizzacount)

# Main Django entry point for the search page 
# https://sumnerhayes.com/dojo/meetupsearch ; this would go in urls.py too
def meetup_search(request):
    return render_to_response("dojo/meetupsearch.html", {'searchform':NameForm()})

# Graphing stuff, using bokeh http://bokeh.pydata.org/en/latest/
from bokeh.resources import CDN
from bokeh.embed import components
from bokeh.plotting import figure, show, output_file, vplot
 
def pizza_tracker(request, date, rsvp, actual, estimate, pizzas):
    plot = figure(plot_width=800, plot_height=400, title="Historical Meetup Attendance (red=actual, navy=estimated, pink=RSVP, green=# of pizzas estimated)")
    plot.multi_line([date]*4, [actual,estimate,rsvp,pizzas],
             line_color=["firebrick", "navy", "pink","green"],line_width=4)
 
    script1, div1 = components(plot, CDN)
 
    return render(request, "dojo/meetup.html", {"the_script1": script1, "the_div1": div1, "pizzanext":int(pizzas[-1]), "rsvpnext":rsvp[-1]})
